import java.io.IOException;
import org.apache.hadoop.conf.Configuration;
import org.apache.hadoop.hbase.HBaseConfiguration;
import org.apache.hadoop.hbase.HTableDescriptor;
import org.apache.hadoop.hbase.MasterNotRunningException;
import org.apache.hadoop.hbase.TableName;
import org.apache.hadoop.hbase.ZooKeeperConnectionException;
import org.apache.hadoop.hbase.client.HBaseAdmin;
import org.apache.hadoop.hbase.client.Scan;
import org.apache.hadoop.hbase.client.coprocessor.AggregationClient;
import org.apache.hadoop.hbase.client.coprocessor.LongColumnInterpreter;
import org.apache.hadoop.hbase.util.Bytes;

public class CoprocessorDemo {
	static Configuration conf = null;
	static{
		conf = HBaseConfiguration.create();
		conf.set("hbase.rootdir", " hdfs://192.168.254.128:9000/hbase ");
		conf.set("hbase.master", "hdfs:// 192.168.254.128:60000");
		conf.set("hbase.zookeeper.property.clientPort", "2181");
		conf.set("hbase.zookeeper.quorum", "master,slave1,slave2");
	}
	
	public static void addTableCoprocessor(String tableName, String coprocessorClassName) {
		try {
				HBaseAdmin admin = new HBaseAdmin(conf); 
				admin.disableTable(tableName);
				HTableDescriptor htd = admin.getTableDescriptor(Bytes.toBytes(tableName));
				htd.addCoprocessor(coprocessorClassName);
				admin.modifyTable(Bytes.toBytes(tableName), htd);
				admin.enableTable(tableName);
		} catch (IOException e) {
			e.printStackTrace();
			System.out.println("error");
		}
	}
		
	public static long rowCount(String tableName, String family) {
		AggregationClient ac = new AggregationClient(conf);  
		Scan scan = new Scan();
		scan.addFamily(Bytes.toBytes(family));
		long rowCount = 0;
		try {
			rowCount = ac.rowCount(TableName.valueOf(tableName), new LongColumnInterpreter(), scan);
		} catch (Throwable e) {
			e.printStackTrace();
			System.out.println("error");
		}  
		return rowCount;
	}

	public static void main(String[] args) throws MasterNotRunningException, ZooKeeperConnectionException, IOException {
		String coprocessorClassName = "org.apache.hadoop.hbase.coprocessor.AggregateImplementation";
		CoprocessorDemo.addTableCoprocessor("stu6", coprocessorClassName);
		long rowCount = CoprocessorDemo.rowCount("stu6", "info");
		HBaseAdmin admin = new HBaseAdmin(conf); 
		System.out.println("rowCount: " + rowCount);
	}
}