package com.mr;

import java.io.DataInput;
import java.io.DataOutput;
import java.io.IOException;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import org.apache.hadoop.conf.Configuration;
import org.apache.hadoop.hbase.HBaseConfiguration;
import org.apache.hadoop.hbase.client.Connection;
import org.apache.hadoop.io.LongWritable;
import org.apache.hadoop.io.Text;
import org.apache.hadoop.io.Writable;
import org.apache.hadoop.mapreduce.Job;
import org.apache.hadoop.mapreduce.Mapper;
import org.apache.hadoop.mapreduce.Reducer;
import org.apache.hadoop.mapreduce.lib.db.DBConfiguration;
import org.apache.hadoop.mapreduce.lib.db.DBInputFormat;
import org.apache.hadoop.mapreduce.lib.db.DBOutputFormat;
import org.apache.hadoop.mapreduce.lib.db.DBWritable;

public class MysqlToMR {
    static Configuration conf = null;
    static Connection connection = null;
    static{
        conf = HBaseConfiguration.create();
        conf.set("hbase.rootdir", " hdfs://192.168.254.128:9000/hbase ");
        conf.set("hbase.master", "hdfs:// 192.168.254.128:60000");
        conf.set("hbase.zookeeper.property.clientPort", "2181");
        conf.set("hbase.zookeeper.quorum", "master,slave1,slave2");
        conf.set("mapred.textoutputformat.separator", ",");
        conf.set("mapred.compress.map.output", "true");
    }

    public static class StudentinfoRecord implements Writable, DBWritable {
        int id;
        String name;
        String sex;
        int age;

        public StudentinfoRecord() {

        }

        public String toString() {
            return new String(this.id + " " + this.name + " " + this.sex + " " + this.age);
        }

        @Override
        public void readFields(ResultSet result) throws SQLException {
            this.id = result.getInt(1);
            this.name = result.getString(2);
            this.sex = result.getString(3);
            this.age = result.getInt(4);
        }

        @Override
        public void write(PreparedStatement stmt) throws SQLException {
            stmt.setInt(1, this.id);
            stmt.setString(2, this.name);
            stmt.setString(3, this.sex);
            stmt.setInt(4, this.age);
        }

        @Override
        public void readFields(DataInput in) throws IOException {
            this.id = in.readInt();
            this.name = Text.readString(in);
            this.sex = Text.readString(in);
            this.age = in.readInt();
        }

        @Override
        public void write(DataOutput out) throws IOException {
            out.writeInt(this.id);
            Text.writeString(out, this.name);
            Text.writeString(out, this.sex);
            out.writeInt(this.age);
        }
    }

    // 记住此处是静态内部类，要不然你自己实现无参构造器，或者等着抛异常：
    public static class DBInputMapper extends
            Mapper<LongWritable, StudentinfoRecord, LongWritable, Text> {
        @Override
        public void map(LongWritable key, StudentinfoRecord value,
                        Context context) throws IOException, InterruptedException {
            context.write(new LongWritable(value.id), new Text(value.toString()));
        }
    }

    public static class MyReducer extends Reducer<LongWritable, Text, StudentinfoRecord, Text> {
        @Override
        public void reduce(LongWritable key, Iterable<Text> values, Context context) throws IOException, InterruptedException {
            String[] splits = values.iterator().next().toString().split(" ");
            StudentinfoRecord r = new StudentinfoRecord();
            r.id = Integer.parseInt(splits[0]);
            r.name = splits[1];
            r.sex = splits[2];
            r.age = Integer.parseInt(splits[3]);
            context.write(r, new Text(r.name));

        }
    }

    public static void main(String[] args) throws IOException, InterruptedException, ClassNotFoundException {

        DBConfiguration.configureDB(conf, "com.mysql.jdbc.Driver","jdbc:mysql://192.168.254.131:3306/hbase","hbase","HBase123@");

        Job job = Job.getInstance(conf, "MysqlToMR");
        job.setJarByClass(MysqlToMR.class);
        job.setMapOutputKeyClass(LongWritable.class);
        job.setMapOutputValueClass(Text.class);
        job.setMapperClass(DBInputMapper.class);
        job.setReducerClass(MyReducer.class);
        job.setOutputKeyClass(LongWritable.class);
        job.setOutputValueClass(Text.class);
        job.setOutputFormatClass(DBOutputFormat.class);
        job.setInputFormatClass(DBInputFormat.class);

        String[] fields = {"id", "name", "sex", "age"};
        // 从 stu 表读数据
        DBInputFormat.setInput(job, StudentinfoRecord.class, "stu", null, null, fields);
        // mapreduce 将数据输出到 stu2 表
        DBOutputFormat.setOutput(job, "stu2", fields);

        job.waitForCompletion(true);
    }
}