package com.mr;

import java.io.IOException;
import java.util.StringTokenizer;
import org.apache.hadoop.conf.Configuration;
import org.apache.hadoop.conf.Configured;
import org.apache.hadoop.hbase.HBaseConfiguration;
import org.apache.hadoop.hbase.HColumnDescriptor;
import org.apache.hadoop.hbase.HTableDescriptor;
import org.apache.hadoop.hbase.MasterNotRunningException;
import org.apache.hadoop.hbase.TableName;
import org.apache.hadoop.hbase.ZooKeeperConnectionException;
import org.apache.hadoop.hbase.client.Admin;
import org.apache.hadoop.hbase.client.Connection;
import org.apache.hadoop.hbase.client.ConnectionFactory;
import org.apache.hadoop.hbase.client.Put;
import org.apache.hadoop.hbase.client.Table;
import org.apache.hadoop.hbase.io.ImmutableBytesWritable;
import org.apache.hadoop.hbase.util.Bytes;
import org.apache.hadoop.io.IntWritable;
import org.apache.hadoop.io.LongWritable;
import org.apache.hadoop.io.Text;
import org.apache.hadoop.io.Writable;
import org.apache.hadoop.mapreduce.Job;
import org.apache.hadoop.mapreduce.Mapper;
import org.apache.hadoop.mapreduce.lib.input.FileInputFormat;
import org.apache.hadoop.hbase.mapreduce.MultiTableOutputFormat;

public class MultipleTablesWriteToHBase extends Configured {

    static Configuration conf = null;
    static Connection connection = null;
    static{
        conf = HBaseConfiguration.create();
        conf.set("hbase.rootdir", " hdfs://192.168.254.128:9000/hbase ");
        conf.set("hbase.master", "hdfs:// 192.168.254.128:60000");
        conf.set("hbase.zookeeper.property.clientPort", "2181");
        conf.set("hbase.zookeeper.quorum", "master,slave1,slave2");
    }

    public static class WriteHBaseMapper extends Mapper<LongWritable, Text, ImmutableBytesWritable, Put> {

        public void map(LongWritable key,Text value, Context context) throws IOException, InterruptedException{
            String[] info = value.toString().split("\t");
            String name = info[0];
            String chinese = info[1];
            String english = info[2];
            String math = info[3];

            ImmutableBytesWritable putTable = new ImmutableBytesWritable("chinese".getBytes());
            Put put = new Put(name.getBytes());
            put.addImmutable("info".getBytes(), "grade".getBytes(), chinese.getBytes());
            context.write(putTable, put);

            putTable = new ImmutableBytesWritable("english".getBytes());
            put = new Put(name.getBytes());
            put.addImmutable("info".getBytes(), "grade".getBytes(), english.getBytes());
            context.write(putTable, put);

            putTable = new ImmutableBytesWritable("math".getBytes());
            put = new Put(name.getBytes());
            put.addImmutable("info".getBytes(), "grade".getBytes(), math.getBytes());
            context.write(putTable, put);
        }

    }

    public static void main(String[] args) throws MasterNotRunningException, ZooKeeperConnectionException, IOException, ClassNotFoundException, InterruptedException {
        // TODO Auto-generated method stub

        connection = ConnectionFactory.createConnection(conf);
        Admin admin = connection.getAdmin();
        String tableName = "chinese";
        TableName tn = TableName.valueOf(tableName);
        HTableDescriptor tableDescriptor = new HTableDescriptor(tn);
        HColumnDescriptor columnDescriptor = new HColumnDescriptor("info");
        tableDescriptor.addFamily(columnDescriptor);
        admin.createTable(tableDescriptor);

        tableName = "english";
        tn = TableName.valueOf(tableName);
        tableDescriptor = new HTableDescriptor(tn);
        tableDescriptor.addFamily(columnDescriptor);
        admin.createTable(tableDescriptor);

        tableName = "math";
        tn = TableName.valueOf(tableName);
        tableDescriptor = new HTableDescriptor(tn);
        tableDescriptor.addFamily(columnDescriptor);
        admin.createTable(tableDescriptor);

        Job job = Job.getInstance(conf, "MultipleTablesWriteToHBase");
        job.setJarByClass(MultipleTablesWriteToHBase.class);
        job.setMapperClass(WriteHBaseMapper.class);
//        TableMapReduceUtil.initTableReducerJob(tableName, WriteHBaseReducer.class, job,null,null,null,null,false);

        job.setMapOutputKeyClass(ImmutableBytesWritable.class);
        job.setMapOutputValueClass(Put.class);
        FileInputFormat.addInputPaths(job, "hdfs://master:9000/data/hbase/data2/1.txt");
        job.setOutputFormatClass(MultiTableOutputFormat.class);
        job.setNumReduceTasks(0);
        job.waitForCompletion(true);
    }
}