package com.mr;

import java.io.DataInput;
import java.io.DataOutput;
import java.io.IOException;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import org.apache.hadoop.conf.Configuration;
import org.apache.hadoop.fs.Path;
import org.apache.hadoop.hbase.HBaseConfiguration;
import org.apache.hadoop.hbase.client.Connection;
import org.apache.hadoop.io.LongWritable;
import org.apache.hadoop.io.Text;
import org.apache.hadoop.io.Writable;
import org.apache.hadoop.mapreduce.Job;
import org.apache.hadoop.mapreduce.Mapper;
import org.apache.hadoop.mapreduce.Reducer;
import org.apache.hadoop.mapreduce.lib.db.DBConfiguration;
import org.apache.hadoop.mapreduce.lib.db.DBOutputFormat;
import org.apache.hadoop.mapreduce.lib.db.DBWritable;
import org.apache.hadoop.mapreduce.lib.input.FileInputFormat;
import org.apache.hadoop.mapreduce.lib.input.TextInputFormat;

public class WriteToMysql {
    static Configuration conf = null;
    static Connection connection = null;
    static{
        conf = HBaseConfiguration.create();
        conf.set("hbase.rootdir", " hdfs://192.168.254.128:9000/hbase ");
        conf.set("hbase.master", "hdfs:// 192.168.254.128:60000");
        conf.set("hbase.zookeeper.property.clientPort", "2181");
        conf.set("hbase.zookeeper.quorum", "master,slave1,slave2");
        conf.set("mapred.textoutputformat.separator", ",");
        conf.set("mapred.compress.map.output", "true");
    }

    //需要实现Writable和DBWritable接口才能完成向MySQL数据库的写入操作
    public static class StudentBean implements Writable, DBWritable {
        private String name;
        private String sex;
        private Integer age;
        @Override
        //向数据库执行写入操作，传入参数是PreparedStatement类型
        public void write(PreparedStatement statement) throws SQLException {
            int index = 1;
            statement.setString(index++, this.getName());
            statement.setString(index++, this.getSex());
            statement.setInt(index++, this.getAge());
        }

        @Override
        //从数据库读取数据，参数是结果集ResultSet，
        public void readFields(ResultSet resultSet) throws SQLException {
            // DBOutputFormat.setOutput中设置字段顺序保持一致
            this.name = resultSet.getString(1);
            this.sex = resultSet.getString(2);
            this.age = resultSet.getInt(3);
        }

        @Override
        public void write(DataOutput out) throws IOException {
            // TODO Auto-generated method stub

        }

        @Override
        public void readFields(DataInput in) throws IOException {

        }

        public String getName() {
            return name;
        }
        public void setName(String name) {
            this.name = name;
        }
        public String getSex() {
            return sex;
        }
        public void setSex(String sex) {
            this.sex = sex;
        }
        public Integer getAge() {
            return age;
        }
        public void setAge(Integer age) {
            this.age = age;
        }
    }

    public static class WriteToMysqlMapper extends  Mapper<LongWritable, Text, Text, Text>{
        @Override
        protected void map(LongWritable key, Text value,
                           Context context)
                throws IOException, InterruptedException {
            //lucy,female,16
            String line = value.toString();
            String fileds[] = line.split("\t");
            String keyString = "";
            String valueString = "";
            try{
                keyString = fileds[0];
                valueString = fileds[1]+","+fileds[2];
            }catch(Exception e){
                e.printStackTrace();
                return;
            }
            context.write(new Text(keyString), new Text(valueString));
        }
    }

    public static class WriteToMysqlReducer extends Reducer<Text, Text, StudentBean, Text>{
        @Override
        public void reduce(Text key, Iterable<Text> values, Context context) throws IOException, InterruptedException  {
            StudentBean bean = new StudentBean();
            String comString = "";
            for (Text val : values) {
                comString = val.toString();
            }
            //lucy,female,16
//			String keyFields[] = key.toString().split(",");
            String valueFields[] = comString.split(",");
            bean.setName(key.toString());
            bean.setSex(valueFields[0]);
            bean.setAge(Integer.parseInt(valueFields[1]));

            context.write(bean,null);
        }
    }

    public static void main(String[] args) throws Exception {
        DBConfiguration.configureDB(conf, "com.mysql.jdbc.Driver","jdbc:mysql://192.168.254.131:3306/hbase","hbase","HBase123@");
        String input="hdfs://master:9000/data/hbase/data4/1.txt";
        Job job = Job.getInstance(conf, "WriteToMysql");
        job.setJarByClass(WriteToMysql.class);
        job.setMapperClass(WriteToMysqlMapper.class);
        job.setReducerClass(WriteToMysqlReducer.class);
        job.setOutputKeyClass(Text.class);
        job.setOutputValueClass(Text.class);
        job.setInputFormatClass(TextInputFormat.class);
        job.setOutputFormatClass(DBOutputFormat.class);
        FileInputFormat.addInputPath(job, new Path(input));

        DBOutputFormat.setOutput(job, "stuInfo",
                "name","sex","age"); // 后面的这些就是数据库表的字段名
        job.waitForCompletion(true);
    }

}
